@@ -19,6 +19,7 @@ def _setup_args(parser: ArgumentParser) -> None:
 
     parser.add_argument("-d", "--debug", action="store_true", help="Print debug to the screen")
     parser.add_argument("-v", "--verbose", action="count", default=0, help="Set verbosity level")
+    parser.add_argument("-t", "--timeout", type=int, help="Set the timeout for a request. 0 = infinite")
     parser.add_argument("--version", action="store_true", help="Print the version number and exit")
 
     sub_parsers = parser.add_subparsers(
@@ -118,23 +119,32 @@ async def main() -> None:
     _LOGGER = logging.getLogger("pyvelop.cli")
     if args.debug:
         _LOGGER.setLevel(logging.DEBUG)
-        _LOGGER.debug(f"Arguments: {args.__dict__}")
+        _LOGGER.debug("Arguments: %s", args.__dict__)
         if args.verbose > 0:
-            logging.getLogger("pyvelop").setLevel(logging.DEBUG)
+            logging.getLogger("pyvelop.mesh").setLevel(logging.DEBUG)
+            logging.getLogger("pyvelop.mesh.verbose").setLevel(logging.INFO)
+            if args.verbose > 1:
+                logging.getLogger("pyvelop.mesh.verbose").setLevel(logging.DEBUG)
     # endregion
 
     if args.version:
         print(_PACKAGE_VERSION)
     else:
-        async with Mesh(node=args.address, username=args.username, password=args.password) as _mesh:
+        async with Mesh(
+                node=args.address,
+                username=args.username,
+                password=args.password,
+                request_timeout=args.timeout,
+        ) as _mesh:
             try:
+                _LOGGER.debug("Gathering details about the Velop system")
                 await _mesh.async_gather_details()
             except MeshInvalidCredentials:
                 _LOGGER.error("Invalid Credentials")
             except MeshBadResponse:
-                _LOGGER.error(f"Bad response received.  Are you sure {args.address} is a Velop node?")
+                _LOGGER.error("Bad response received.  Are you sure %s is a Velop node?", args.address)
             except MeshNodeNotPrimary:
-                _LOGGER.error(f"{args.address} is not the primary node")
+                _LOGGER.error("%s is not the primary node", args.address)
             else:
                 if args.target == "mesh":
                     # region #-- get the node names --#
@@ -148,7 +158,7 @@ async def main() -> None:
 
                     # region #-- get WAN details --#
                     if args.get_wan or all_args:
-                        _LOGGER.debug("Preparing WAND details")
+                        _LOGGER.debug("Preparing WAN details")
                         section = "WAN Details"
                         section += f"\n{'-' * len(section)}\n"
                         section += f"Connected: {_mesh.wan_status}\n"\
