@@ -4,24 +4,32 @@ import base64
 import json
 import logging
 import time
+from asyncio.exceptions import TimeoutError
 from typing import Union, List
 
 import aiohttp
+from aiohttp.client_exceptions import (
+    ClientConnectionError,
+    ClientConnectorError,
+)
 
 from . import const
 from .device import Device
 from .exceptions import (
     MeshBadResponse,
+    MeshConnectionError,
     MeshDeviceNotFoundResponse,
     MeshInvalidArguments,
     MeshInvalidCredentials,
     MeshInvalidInput,
     MeshNodeNotPrimary,
+    MeshTimeoutError,
     MeshTooManyMatches,
 )
 from .node import Node
 
 _LOGGER = logging.getLogger(__name__)
+_LOGGER_VERBOSE = logging.getLogger(f"{__name__}.verbose")
 
 
 def _get_action_index(action: str, payload: List[dict]) -> Union[int, None]:
@@ -107,6 +115,29 @@ def _process_speedtest_results(speedtest_results=None, only_latest: bool = False
     return ret
 
 
+def _get_speedtest_state(speedtest_results=None) -> str:
+    """Process the Speedtest results to get a textual state"""
+
+    if speedtest_results is None:
+        speedtest_results = {}
+
+    if speedtest_results:
+        if speedtest_results.get("uploadBandwidth", 0):
+            ret = "Checking upload speed"
+        elif speedtest_results.get("downloadBandwidth", 0):
+            ret = "Checking download speed"
+        elif speedtest_results.get("latency"):
+            ret = "Checking latency"
+        elif speedtest_results.get("serverID", "") == '0':
+            ret = "Detecting server"
+        else:
+            ret = ""
+    else:
+        ret = ""
+
+    return ret
+
+
 def _process_raw_device_results(device_results=None) -> None:
     """Add the required details to the device results.
 
@@ -131,14 +162,18 @@ class Mesh:
     If you need live information then call the corresponding method.
     """
 
-    def __init__(self, node: str, password: str, username: str = "admin"):
+    def __init__(self, node: str, password: str, username: str = "admin", request_timeout: Union[int, None] = None):
         """Constructor
 
         :param node: The node we should make a connection to
         :param password: password to use
-        :param username: username to use
+        :param username: username to use; default admin
+        :param request_timeout: number of seconds to timeout the request; default 10s
         """
 
+        if request_timeout is None:
+            request_timeout = 10
+
         self._session: aiohttp.ClientSession
 
         self.__mesh_attributes: dict = {  # initialise the attributes for the mesh
@@ -156,9 +191,12 @@ class Mesh:
         self.__api_url: str = f"http://{self.__mesh_attributes[const.ATTR_MESH_CONNECTED_NODE]}/JNAP/"
         self.__username: str = username
         self.__password: str = password
+        self.__timeout: int = request_timeout
         self.__create_session()
 
-        _LOGGER.debug(f"Initialised mesh for {self.__mesh_attributes[const.ATTR_MESH_CONNECTED_NODE]}")
+        # noinspection PyProtectedMember
+        _LOGGER.debug("%s version: %s", __package__, const._PACKAGE_VERSION)
+        _LOGGER.debug("Initialised mesh for %s", self.__mesh_attributes[const.ATTR_MESH_CONNECTED_NODE])
 
     async def __aenter__(self):
         """Asynchronous enter magic method"""
@@ -187,15 +225,28 @@ class Mesh:
         :return: THe JSON response or raises an error if need be
         """
 
+        _LOGGER_VERBOSE.debug(
+            "URL: %s, Action: %s, Payload: %s, Timeout: %i",
+            self.__api_url,
+            action,
+            json.dumps(payload),
+            self.__timeout
+        )
+
         if payload is None:
             payload = []
 
         headers = self.__get_headers()
         headers["X-JNAP-Action"] = action
         try:
-            if self._session.closed:
+            if self._session.closed:  # session closed so recreate it
+                _LOGGER_VERBOSE.debug("Session was closed.")
                 self.__create_session()
-            resp = await self._session.post(url=self.__api_url, headers=headers, json=payload)
+            resp = await self._session.post(url=self.__api_url, headers=headers, json=payload, timeout=self.__timeout)
+        except TimeoutError:
+            raise MeshTimeoutError
+        except (ClientConnectionError, ClientConnectorError,):
+            raise MeshConnectionError
         except aiohttp.ClientError:
             raise
         else:
@@ -204,14 +255,16 @@ class Mesh:
             except aiohttp.ClientError:
                 raise MeshBadResponse
             else:
+                _LOGGER_VERBOSE.debug("Response: %s", json.dumps(resp_json))
                 if _is_valid_response(response=resp_json):
                     ret = resp_json
                 else:  # process API specific errors
-                    err = MeshBadResponse
                     if "responses" not in resp_json:
                         resp_json = {"responses": [resp_json]}
 
+                    err = None
                     for resp in resp_json.get("responses", []):
+                        err = None
                         if resp.get("result") == "_ErrorInvalidInput":
                             err = MeshInvalidInput(resp.get("error"))
                         elif resp.get("result") == "_ErrorUnauthorized":
@@ -221,22 +274,23 @@ class Mesh:
                             err = MeshInvalidInput("Unknown JNAP Action")
                         elif resp.get("result") == "ErrorDeviceNotInMasterMode":
                             err = MeshNodeNotPrimary
-                        elif not resp.get("result").startswith("_"):
+                        elif resp.get("result") != "OK" and not resp.get("result").startswith("_"):
                             err = MeshInvalidInput(resp.get("result"))
 
+                        if err:
+                            break
+
+                    if not err:
+                        _LOGGER.error("Unknown error received: %s", json.dumps(resp_json))
+                        err = MeshBadResponse
+
                     raise err
 
         return ret
 
     async def __async_gather_details(
             self,
-            include_backhaul: bool = False,
-            include_devices: bool = False,
-            include_firmware_update: bool = False,
-            include_guest_wifi: bool = False,
-            include_parental_control: bool = False,
-            include_speedtest_results: bool = False,
-            include_wan: bool = False,
+            **kwargs,
     ) -> dict:
         """Work is done here to gather the necessary details for mesh.
 
@@ -249,194 +303,211 @@ class Mesh:
         :return: A dictionary containing the relevant details.  Keys used will match those of the instance variable.
         """
 
-        try:
-            await self.async_test_credentials()
-        except Exception as err:
-            raise err
-        else:
-            ret = {}
-            payload: List = []
+        _LOGGER.debug("Gathering details: %s", json.dumps(kwargs))
 
-            # -- get the devices --#
-            if include_devices:
-                payload.append({
-                    "action": const.ACTION_JNAP_GET_DEVICES,
-                    "request": {},
-                })
+        ret = {}
+        payload: List = []
 
-            # -- get the backhaul info  --#
-            if include_backhaul:
-                payload.append({
-                    "action": const.ACTION_JNAP_GET_BACKHAUL,
-                    "request": {},
-                })
+        # -- get the devices --#
+        if kwargs.get("include_devices"):
+            payload.append({
+                "action": const.ACTION_JNAP_GET_DEVICES,
+                "request": {},
+            })
 
-            # -- get the guest WiFi details --#
-            if include_guest_wifi:
-                payload.append({
-                    "action": const.ACTION_JNAP_GET_GUEST_NETWORK_INFO,
-                    "request": {},
-                })
+        # -- get the backhaul info  --#
+        if kwargs.get("include_backhaul"):
+            payload.append({
+                "action": const.ACTION_JNAP_GET_BACKHAUL,
+                "request": {},
+            })
 
-            # -- get the Parental Control details --#
-            if include_parental_control:
-                payload.append({
-                    "action": const.ACTION_JNAP_GET_PARENTAL_CONTROL_INFO,
-                    "request": {},
-                })
+        # -- get the guest WiFi details --#
+        if kwargs.get("include_guest_wifi"):
+            payload.append({
+                "action": const.ACTION_JNAP_GET_GUEST_NETWORK_INFO,
+                "request": {},
+            })
 
-            # -- get the latest Speedtest result --#
-            if include_speedtest_results:
-                payload.append({
-                    "action": const.ACTION_JNAP_GET_SPEEDTEST_RESULTS,
-                    "request": {**const.DEF_JNAP_SPEEDTEST_PAYLOAD, "lastNumberOfResults": 10},
-                })
+        # -- get the Parental Control details --#
+        if kwargs.get("include_parental_control"):
+            payload.append({
+                "action": const.ACTION_JNAP_GET_PARENTAL_CONTROL_INFO,
+                "request": {},
+            })
 
-            # -- get the update check details --#
-            if include_firmware_update:
-                payload.append({
-                    "action": const.ACTION_JNAP_GET_UPDATE_FIRMWARE_STATE,
-                    "request": {},
-                })
+        # -- get the current Speedtest state --#
+        if kwargs.get("include_speedtest_state"):
+            payload.append({
+                "action": const.ACTION_JNAP_GET_SPEEDTEST_STATE,
+                "request": {}
+            })
 
-            # -- get the WAN details --#
-            if include_wan:
-                payload.append({
-                    "action": const.ACTION_JNAP_GET_WAN_INFO,
-                    "request": {},
-                })
+        # -- get the latest Speedtest result --#
+        if kwargs.get("include_speedtest_results"):
+            payload.append({
+                "action": const.ACTION_JNAP_GET_SPEEDTEST_RESULTS,
+                "request": {**const.DEF_JNAP_SPEEDTEST_PAYLOAD, "lastNumberOfResults": 10},
+            })
 
-            resp = await self.__async_make_request(action=const.ACTION_JNAP_TRANSACTION, payload=payload)
-            responses = resp.get("responses", [])
-            if responses:
-                # region #-- populate device and node details --#
-                idx = _get_action_index(
-                    action=const.ACTION_JNAP_GET_DEVICES,
-                    payload=payload
-                )
-                device_info: List[dict] = []
-                if idx is not None:
-                    device_info = responses[idx] \
-                        .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {}) \
-                        .get("devices", [])
-                    _process_raw_device_results(device_results=device_info)
+        # -- get the update check details --#
+        if kwargs.get("include_firmware_update"):
+            payload.append({
+                "action": const.ACTION_JNAP_GET_UPDATE_FIRMWARE_STATE,
+                "request": {},
+            })
 
-                idx = _get_action_index(
-                    action=const.ACTION_JNAP_GET_BACKHAUL,
-                    payload=payload
-                )
-                backhaul_info: dict = {}
-                if idx is not None:
-                    backhaul_info = responses[idx] \
-                        .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {}) \
-                        .get("backhaulDevices", [])
+        # -- get the WAN details --#
+        if kwargs.get("include_wan"):
+            payload.append({
+                "action": const.ACTION_JNAP_GET_WAN_INFO,
+                "request": {},
+            })
 
-                # region #-- build the properties for the device types --#
-                devices = []
-                for device in device_info:
-                    if "nodeType" in device:
-                        device_backhaul = [bi for bi in backhaul_info if bi.get("deviceUUID") == device.get("deviceID")]
-                        if device_backhaul:
-                            device_backhaul = device_backhaul[0]
-                        else:
-                            device_backhaul = {}
-                        n = Node(**device, **{"backhaul": device_backhaul})
-                        devices.append(n)
+        resp = await self.__async_make_request(action=const.ACTION_JNAP_TRANSACTION, payload=payload)
+        responses = resp.get("responses", [])
+        if responses:
+            # region #-- populate device and node details --#
+            idx = _get_action_index(
+                action=const.ACTION_JNAP_GET_DEVICES,
+                payload=payload
+            )
+            device_info: List[dict] = []
+            if idx is not None:
+                device_info = responses[idx] \
+                    .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {}) \
+                    .get("devices", [])
+                _process_raw_device_results(device_results=device_info)
+
+            idx = _get_action_index(
+                action=const.ACTION_JNAP_GET_BACKHAUL,
+                payload=payload
+            )
+            backhaul_info: dict = {}
+            if idx is not None:
+                backhaul_info = responses[idx] \
+                    .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {}) \
+                    .get("backhaulDevices", [])
+
+            # region #-- build the properties for the device types --#
+            devices = []
+            for device in device_info:
+                if "nodeType" in device:
+                    device_backhaul = [bi for bi in backhaul_info if bi.get("deviceUUID") == device.get("deviceID")]
+                    if device_backhaul:
+                        device_backhaul = device_backhaul[0]
                     else:
-                        d = Device(**device)
-                        devices.append(d)
-                # endregion
+                        device_backhaul = {}
+                    n = Node(**device, **{"backhaul": device_backhaul})
+                    devices.append(n)
+                else:
+                    d = Device(**device)
+                    devices.append(d)
+            # endregion
 
-                # region #-- calculate the connected devices for nodes and parent name for devices --#
-                for node in devices:
-                    if node.__class__.__name__.lower() == "node":
-                        connected_devices: List = []
-                        for device in devices:
-                            for adapter in device.network:
-                                if adapter.get("parent_id") == node.unique_id:
-                                    connected_devices.append({"name": device.name, "ip": adapter.get("ip")})
-                                if node.parent_ip:
-                                    if node.parent_ip == adapter.get("ip"):
-                                        setattr(node, "parent_name", device.name)
-                                else:
-                                    setattr(node, "parent_name", None)
-                        setattr(node, "_Node__connected_devices", connected_devices)
-                    elif node.__class__.__name__.lower() == "device":
-                        attrib_connections = getattr(node, "_Device__attributes", {}).get("connections", [])
-                        parent: Union[str, None] = None
-                        for conn in attrib_connections:
-                            if conn.get("parentDeviceID", ""):
-                                try:
-                                    parent = [
-                                        device.name
-                                        for device in devices
-                                        if device.unique_id == conn.get("parentDeviceID")
-                                    ][0]
-                                except IndexError:
-                                    pass
-                        setattr(node, "parent_name", parent)
-                # endregion
+            # region #-- calculate the connected devices for nodes and parent name for devices --#
+            for node in devices:
+                if node.__class__.__name__.lower() == "node":
+                    connected_devices: List = []
+                    for device in devices:
+                        for adapter in device.network:
+                            if adapter.get("parent_id") == node.unique_id:
+                                connected_devices.append({"name": device.name, "ip": adapter.get("ip")})
+                            if node.parent_ip:
+                                if node.parent_ip == adapter.get("ip"):
+                                    setattr(node, "parent_name", device.name)
+                            else:
+                                setattr(node, "parent_name", None)
+                    setattr(node, "_Node__connected_devices", connected_devices)
+                elif node.__class__.__name__.lower() == "device":
+                    attrib_connections = getattr(node, "_Device__attributes", {}).get("connections", [])
+                    parent: Union[str, None] = None
+                    for conn in attrib_connections:
+                        if conn.get("parentDeviceID", ""):
+                            try:
+                                parent = [
+                                    device.name
+                                    for device in devices
+                                    if device.unique_id == conn.get("parentDeviceID")
+                                ][0]
+                            except IndexError:
+                                pass
+                    setattr(node, "parent_name", parent)
+            # endregion
 
-                if devices:
-                    ret[const.ATTR_MESH_DEVICES] = devices
-                # endregion
+            if devices:
+                ret[const.ATTR_MESH_DEVICES] = devices
+            # endregion
 
-                # region #-- populate the Guest Wi-Fi details --#
-                idx = _get_action_index(
-                    action=const.ACTION_JNAP_GET_GUEST_NETWORK_INFO,
-                    payload=payload
+            # region #-- populate the Guest Wi-Fi details --#
+            idx = _get_action_index(
+                action=const.ACTION_JNAP_GET_GUEST_NETWORK_INFO,
+                payload=payload
+            )
+            if idx is not None:
+                ret[const.ATTR_MESH_GUEST_NETWORK_INFO] = responses[idx] \
+                    .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
+            # endregion
+
+            # region #-- populate the Parental Control details --#
+            idx = _get_action_index(
+                action=const.ACTION_JNAP_GET_PARENTAL_CONTROL_INFO,
+                payload=payload
+            )
+            if idx is not None:
+                ret[const.ATTR_MESH_PARENTAL_CONTROL_INFO] = responses[idx] \
+                    .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
+            # endregion
+
+            # region #-- populate the WAN details --#
+            idx = _get_action_index(
+                action=const.ACTION_JNAP_GET_WAN_INFO,
+                payload=payload
+            )
+            if idx is not None:
+                ret[const.ATTR_MESH_WAN_INFO] = responses[idx] \
+                    .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
+            # endregion
+
+            # region #-- populate the latest Speedtest results --#
+            idx = _get_action_index(
+                action=const.ACTION_JNAP_GET_SPEEDTEST_RESULTS,
+                payload=payload
+            )
+            if idx is not None:
+                speedtest_results = responses[idx] \
+                    .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {}) \
+                    .get("healthCheckResults", [])
+                speedtest_results = _process_speedtest_results(
+                    speedtest_results,
+                    only_completed=True,
+                    only_latest=True
                 )
-                if idx is not None:
-                    ret[const.ATTR_MESH_GUEST_NETWORK_INFO] = responses[idx] \
-                        .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
-                # endregion
+                ret[const.ATTR_MESH_SPEEDTEST_RESULTS] = speedtest_results
+            # endregion
 
-                # region #-- populate the Parental Control details --#
-                idx = _get_action_index(
-                    action=const.ACTION_JNAP_GET_PARENTAL_CONTROL_INFO,
-                    payload=payload
+            # region #-- populate the current Speedtest status --#
+            idx = _get_action_index(
+                action=const.ACTION_JNAP_GET_SPEEDTEST_STATE,
+                payload=payload
+            )
+            if idx is not None:
+                ret[const.ATTR_MESH_SPEEDTEST_STATE] = _get_speedtest_state(
+                    speedtest_results=responses[idx]
+                    .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
+                    .get("speedTestResult", {})
                 )
-                if idx is not None:
-                    ret[const.ATTR_MESH_PARENTAL_CONTROL_INFO] = responses[idx] \
-                        .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
-                # endregion
+            # endregion
 
-                # region #-- populate the WAN details --#
-                idx = _get_action_index(
-                    action=const.ACTION_JNAP_GET_WAN_INFO,
-                    payload=payload
-                )
-                if idx is not None:
-                    ret[const.ATTR_MESH_WAN_INFO] = responses[idx] \
-                        .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
-                # endregion
-
-                # region #-- populate the latest Speedtest results --#
-                idx = _get_action_index(
-                    action=const.ACTION_JNAP_GET_SPEEDTEST_RESULTS,
-                    payload=payload
-                )
-                if idx is not None:
-                    speedtest_results = responses[idx] \
-                        .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {}) \
-                        .get("healthCheckResults", [])
-                    speedtest_results = _process_speedtest_results(
-                        speedtest_results,
-                        only_completed=True,
-                        only_latest=True
-                    )
-                    ret[const.ATTR_MESH_SPEEDTEST_RESULTS] = speedtest_results
-                # endregion
-
-                # region #-- populate the update check details --#
-                idx = _get_action_index(
-                    action=const.ACTION_JNAP_GET_UPDATE_FIRMWARE_STATE,
-                    payload=payload
-                )
-                if idx is not None:
-                    ret[const.ATTR_MESH_UPDATE_FIRMWARE_STATE] = responses[idx] \
-                        .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
-                # endregion
+            # region #-- populate the update check details --#
+            idx = _get_action_index(
+                action=const.ACTION_JNAP_GET_UPDATE_FIRMWARE_STATE,
+                payload=payload
+            )
+            if idx is not None:
+                ret[const.ATTR_MESH_UPDATE_FIRMWARE_STATE] = responses[idx] \
+                    .get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
+            # endregion
 
         return ret
 
@@ -451,7 +522,7 @@ class Mesh:
         if radios is None:
             radios = []
 
-        _LOGGER.debug(f"Setting the guest Wi-Fi to: {'on' if state else 'off'}")
+        _LOGGER.debug("Setting the guest Wi-Fi to: %s", 'on' if state else 'off')
         payload = {
             "isGuestNetworkEnabled": state,
             "radios": radios,
@@ -469,7 +540,7 @@ class Mesh:
         if rules is None:
             rules = []
 
-        _LOGGER.debug(f"Setting parental controls to: {'on' if state else 'off'}")
+        _LOGGER.debug("Setting parental controls to: %s", 'on' if state else 'off')
         payload = {
             "isParentalControlEnabled": state,
             "rules": rules,
@@ -484,6 +555,7 @@ class Mesh:
         :return: None
         """
 
+        _LOGGER_VERBOSE.debug("Creating session.")
         self._session = aiohttp.ClientSession(raise_for_status=True)
 
     def __credentials(self) -> str:
@@ -513,7 +585,7 @@ class Mesh:
         :return: None
         """
 
-        _LOGGER.debug(f"Initiating a check for new firmware")
+        _LOGGER.debug("Initiating a check for new firmware")
 
         await self.__async_make_request(
             action=const.ACTION_JNAP_UPDATE_FIRMWARE,
@@ -533,7 +605,7 @@ class Mesh:
         :return: None
         """
 
-        _LOGGER.debug(f"Deleting device: {kwargs}")
+        _LOGGER.debug("Deleting device: %s", kwargs)
 
         device_id: str
         if "device_id" in kwargs:
@@ -574,25 +646,31 @@ class Mesh:
             include_speedtest_results=True,
             include_wan=True,
             include_firmware_update=True,
+            include_speedtest_state=True,
         )
 
         # region #-- split the devices into their types --#
+        _LOGGER.debug("Populating nodes")
         self.__mesh_attributes[const.ATTR_MESH_NODES] = [
             device
             for device in details[const.ATTR_MESH_DEVICES]
             if device.__class__.__name__.lower() == "node"
         ]
+        _LOGGER.debug("Populated %i nodes", len(self.__mesh_attributes[const.ATTR_MESH_NODES]))
 
+        _LOGGER.debug("Populating devices")
         self.__mesh_attributes[const.ATTR_MESH_DEVICES] = [
             device
             for device in details.get(const.ATTR_MESH_DEVICES, [])
             if device.__class__.__name__.lower() == "device"
         ]
+        _LOGGER.debug("Populated %i devices", len(self.__mesh_attributes[const.ATTR_MESH_DEVICES]))
         # endregion
 
         # region #-- manage the other attributes --#
         details.pop(const.ATTR_MESH_DEVICES)
         for attr in details:
+            _LOGGER_VERBOSE.debug("Populating %s", attr)
             self.__mesh_attributes[attr] = details[attr]
         # endregion
 
@@ -608,6 +686,8 @@ class Mesh:
         :return: Device or Node object whichever is applicable
         """
 
+        _LOGGER.debug("Getting device for ID: %s (force_refresh=%s)", device_id, force_refresh)
+
         all_devices: List[Union[Device, Node]]
         if not force_refresh:
             all_devices = self.devices + self.nodes
@@ -641,6 +721,8 @@ class Mesh:
         :return:  Device or Node object whichever is applicable
         """
 
+        _LOGGER.debug("Getting device for AMC: %s (force_refresh=%s)", mac_address, force_refresh)
+
         # noinspection PyTypeChecker
         ret: Union[Device, Node] = None
 
@@ -674,6 +756,8 @@ class Mesh:
         :return: List of device objects
         """
 
+        _LOGGER.debug("Getting devices from the API")
+
         all_devices = await self.__async_gather_details(
             include_devices=True,
         )
@@ -700,7 +784,7 @@ class Mesh:
         :return: List of dictionaries containing the result details
         """
 
-        _LOGGER.debug("Gathering SpeedTest results")
+        _LOGGER.debug("Gathering Speedtest results: %s")
 
         payload = {**const.DEF_JNAP_SPEEDTEST_PAYLOAD, "lastNumberOfResults": count}
         resp = await self.__async_make_request(action=const.ACTION_JNAP_GET_SPEEDTEST_RESULTS, payload=payload)
@@ -720,21 +804,14 @@ class Mesh:
         :return: A string containing the stage
         """
 
-        resp = await self.__async_make_request(action=const.ACTION_JNAP_GET_SPEEDTEST_STATE)
-        output: dict = resp.get(const.KEY_ACTION_JNAP_RESPONSE_RESULTS, {})
-        if output:
-            output = output.get("speedTestResult", {})
+        _LOGGER.debug("Getting the current state of the Speedtest")
 
-        if output.get("uploadBandwidth", 0):
-            ret = "Checking upload speed"
-        elif output.get("downloadBandwidth", 0):
-            ret = "Checking download speed"
-        elif output.get("latency"):
-            ret = "Checking latency"
-        elif output.get("serverID", "") == '0':
-            ret = "Detecting server"
-        else:
-            ret = ""
+        resp = await self.__async_gather_details(
+            include_speedtest_state=True,
+        )
+        ret = resp[const.ATTR_MESH_SPEEDTEST_STATE]
+
+        _LOGGER.debug("Speedtest state: %s", ret)
 
         return ret
 
@@ -744,13 +821,19 @@ class Mesh:
         :return: True if still running, False if not
         """
 
+        _LOGGER.debug("Getting the current state of the update check")
+
         resp = await self.__async_gather_details(
             include_firmware_update=True
         )
         node_results = resp.get(const.ATTR_MESH_UPDATE_FIRMWARE_STATE, {}).get("firmwareUpdateStatus", [])
         all_states = ["pendingOperation" in node for node in node_results]
 
-        return any(all_states)
+        ret: bool = any(all_states)
+
+        _LOGGER.debug("Update check state: %s", ret)
+
+        return ret
 
     async def async_set_guest_wifi_state(self, state: bool) -> None:
         """Set the state of the guest Wi-Fi.
@@ -809,7 +892,7 @@ class Mesh:
         :return: True if valid, False if not
         """
 
-        _LOGGER.debug(f"Checking credentials against {self.__mesh_attributes[const.ATTR_MESH_CONNECTED_NODE]}")
+        _LOGGER.debug("Checking credentials against %s", self.__mesh_attributes[const.ATTR_MESH_CONNECTED_NODE])
 
         ret = await self.__async_make_request(action=const.ACTION_JNAP_CHECK_PASSWORD)
         ret = True if ret.get("result", False) else False
@@ -822,6 +905,8 @@ class Mesh:
         :return: None
         """
 
+        _LOGGER.debug("Closing session to: %s", self.connected_node)
+
         return await self._session.close()
 
     @property
@@ -904,6 +989,12 @@ class Mesh:
 
         return self.__mesh_attributes[const.ATTR_MESH_PARENTAL_CONTROL_INFO].get("isParentalControlEnabled", False)
 
+    @property
+    def speedtest_status(self) -> str:
+        """Returns the current status of the Speedtest"""
+
+        return self.__mesh_attributes[const.ATTR_MESH_SPEEDTEST_STATE]
+
     @property
     def speedtest_results(self) -> List:
         """Get the Speedtest results
